//
//  AliMITBaseStructDefine.h
//
//  Created by 沐翊（shirley） on 17/8/16.
//  Copyright © 2017年 Taobao.com. All rights reserved.
//
//

#ifndef BaseStructDefine_h
#define BaseStructDefine_h



#include <iostream>
using namespace std;


#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */



#define MAX_COLOR_TABLE_NUM   3  //must >= 3


#define UNKOWN_INPUT_TYPE            -1
#define TEXTURE_ID_AS_INPUT_TYPE     0 //oes texture
#define RAW_DATA_AS_INPUT_TYPE       1
#define USR_ALLOC_TEXTURE_ID_AS_INPUT_TYPE     2

#define UNKOWN_OUTPUT_TYPE           -1
#define USR_FBO_ID_AS_OUTPUT_TYPE    0
#define TEXTURE_ID_AS_OUTPUT_TYPE    1
#define RAW_DATA_AS_OUTPUT_TYPE      2  //注意data内存由调用层管理

typedef enum {
    Format_UNKNOWN = 0,
    Format_YUV420SP_NV12 = 1,   //nv12
    Format_YUV420P = 2,
    Format_RGB = 3,
    Format_RGBA = 4,
    Format_BGRA = 5,
    Format_YUV420SP_NV21 = 6,   //nv21
} RawDataFormat;


typedef struct __imgRawData {

    __imgRawData();
    __imgRawData(int width, int height, int format, long len, bool bIfAllocDataMemory);

    ~__imgRawData();

    // 以下3个接口是老接口，不支持非标准size、有字节对齐处理的yuv格式
    // 建议直接用后面的新接口，不要用以下3个老接口（这里是为了兼容最早接入的业务方的编译，才没有删掉）
    
    //use the same memory, no memory copy:
    int SetValuesWithDataAddress(int width, int height, int format, long len, unsigned char * pData);

    //use different memory, with memory copy:
    int SetValuesWithData(int width, int height, int format, long len, const unsigned char * pData);

    void GetValues(unsigned char ** ppDataAddress,
                   int & width,
                   int & height,
                   long & len,
                   int & format);


    // 以上3个接口没有考虑yuv裸数据的字节对齐问题（在某些图像size的情况下，裸数据一般都经过了系统的字节对齐处理，uv的头地址并不是y地址直接加w*h可以求得，因此uv的头地址需要上层传进来）
    // 建议采用下面的3个新接口（不然在某些图像size的情况下，对于yuv的格式，可能有“uv通道头地址不对导致的花屏”问题）

    //use the same memory, no memory copy:
    int SetValuesWithDataAddress(int iImgWidth, int iImgHeight, int iImgFormat, long lAllPlanesTotalDataLen,
                                 unsigned char * pDataBase, unsigned char * pDataAddressPlane0, unsigned char * pDataAddressPlane1, unsigned char * pDataAddressPlane2);

    //use different memory, with memory copy:
    int SetValuesWithData(int iImgWidth, int iImgHeight, int iImgFormat, long lAllPlanesTotalDataLen,
                          unsigned char * pDataBase, unsigned char * pDataAddressPlane0, unsigned char * pDataAddressPlane1, unsigned char * pDataAddressPlane2);

    void GetValues(int & iImgWidth, int & iImgHeight, int & iImgFormat, long & lAllPlanesTotalDataLen,
                   int & iHowManyPlanes, unsigned char ** ppDataAddressPlane0, unsigned char ** ppDataAddressPlane1, unsigned char ** ppDataAddressPlane2);
    
    
    
    
    int GetWidth();
    int GetHeight();

    int FillWithRGBAColor(unsigned char r, unsigned char g, unsigned char b, unsigned char a);
    int FillWithRGBColor(unsigned char r, unsigned char g, unsigned char b);


private:

    int             iWidth;
    int             iHeight;
    long            lTotalDataLen;
    int             iDataFormat;
    bool            bIfDataMemoryNeedRelease;

    int             iPlaneNum;
    unsigned char * pBaseDataAddress;
    unsigned char * pDataAddressOfPlane0;
    unsigned char * pDataAddressOfPlane1;
    unsigned char * pDataAddressOfPlane2;
    
}ImgRawData;


typedef struct __resourceVertexPosition {
    __resourceVertexPosition()
    {
        bIfAvailable = false;
        fLeftBottomPoint_X = 0.0f;
        fLeftBottomPoint_Y = 0.0f;
        fRightBottomPoint_X = 0.0f;
        fRightBottomPoint_Y = 0.0f;
        fLeftUpPoint_X = 0.0f;
        fLeftUpPoint_Y = 0.0f;
        fRightUpPoint_X = 0.0f;
        fRightUpPoint_Y = 0.0f;
    }
    
    ~__resourceVertexPosition()
    {
        bIfAvailable = false;
        fLeftBottomPoint_X = 0.0f;
        fLeftBottomPoint_Y = 0.0f;
        fRightBottomPoint_X = 0.0f;
        fRightBottomPoint_Y = 0.0f;
        fLeftUpPoint_X = 0.0f;
        fLeftUpPoint_Y = 0.0f;
        fRightUpPoint_X = 0.0f;
        fRightUpPoint_Y = 0.0f;
    }

    __resourceVertexPosition(bool bIfAvailable_copyValue,
                            float fLeftBottomPoint_X_copyValue,
                            float fLeftBottomPoint_Y_copyValue,
                            float fRightBottomPoint_X_copyValue,
                            float fRightBottomPoint_Y_copyValue,
                            float fLeftUpPoint_X_copyValue,
                            float fLeftUpPoint_Y_copyValue,
                            float fRightUpPoint_X_copyValue,
                            float fRightUpPoint_Y_copyValue)
    {
        bIfAvailable = bIfAvailable_copyValue;
        fLeftBottomPoint_X = fLeftBottomPoint_X_copyValue;
        fLeftBottomPoint_Y = fLeftBottomPoint_Y_copyValue;
        fRightBottomPoint_X = fRightBottomPoint_X_copyValue;
        fRightBottomPoint_Y = fRightBottomPoint_Y_copyValue;
        fLeftUpPoint_X = fLeftUpPoint_X_copyValue;
        fLeftUpPoint_Y = fLeftUpPoint_Y_copyValue;
        fRightUpPoint_X = fRightUpPoint_X_copyValue;
        fRightUpPoint_Y = fRightUpPoint_Y_copyValue;
    }

    void SetValues(bool bIfAvailable_copyValue,
              float fLeftBottomPoint_X_copyValue,
              float fLeftBottomPoint_Y_copyValue,
              float fRightBottomPoint_X_copyValue,
              float fRightBottomPoint_Y_copyValue,
              float fLeftUpPoint_X_copyValue,
              float fLeftUpPoint_Y_copyValue,
              float fRightUpPoint_X_copyValue,
              float fRightUpPoint_Y_copyValue)
    {
        bIfAvailable = bIfAvailable_copyValue;
        fLeftBottomPoint_X = fLeftBottomPoint_X_copyValue;
        fLeftBottomPoint_Y = fLeftBottomPoint_Y_copyValue;
        fRightBottomPoint_X = fRightBottomPoint_X_copyValue;
        fRightBottomPoint_Y = fRightBottomPoint_Y_copyValue;
        fLeftUpPoint_X = fLeftUpPoint_X_copyValue;
        fLeftUpPoint_Y = fLeftUpPoint_Y_copyValue;
        fRightUpPoint_X = fRightUpPoint_X_copyValue;
        fRightUpPoint_Y = fRightUpPoint_Y_copyValue;
    }

    
    bool  bIfAvailable;
    float fLeftBottomPoint_X;
    float fLeftBottomPoint_Y;
    float fRightBottomPoint_X;
    float fRightBottomPoint_Y;
    float fLeftUpPoint_X;
    float fLeftUpPoint_Y;
    float fRightUpPoint_X;
    float fRightUpPoint_Y;
}ResourceVertexPosition;


typedef struct __gpuImgInfo {

    __gpuImgInfo()
    {
        iWidth = 0;
        iHeight = 0;
        iID = -1;
        iFormat = Format_UNKNOWN;
    }

    ~__gpuImgInfo()
    {
        iWidth = 0;
        iHeight = 0;
        iID = -1;
        iFormat = Format_UNKNOWN;

    }

    int             iID;
    int             iWidth;
    int             iHeight;
    int             iFormat;

}GpuImgResourceInfo;


typedef enum {
    EFFECT_TYPE_BASIC_YUV = 0,
    
    EFFECT_TYPE_FACE_BEAUTY = 1,  //注意如果有美颜，美颜必须放在第1个effect！！！！！
    
    EFFECT_TYPE_COLOR_STYLE_CHALKBOARD = 101,
    EFFECT_TYPE_COLOR_STYLE_REDGELB = 102,
    EFFECT_TYPE_COLOR_STYLE_NEWWARM = 103,
    EFFECT_TYPE_COLOR_STYLE_COOLCHILDREN = 104,
    
    EFFECT_TYPE_IMAGE_TRANSITION_SHUTTERS = 201,
    EFFECT_TYPE_IMAGE_TRANSITION_CIRCLE = 202,
    EFFECT_TYPE_IMAGE_TRANSITION_LEFTRIGHT = 203,
    EFFECT_TYPE_IMAGE_TRANSITION_FADE = 204,

    EFFECT_TYPE_CHARTLET = 301,

    EFFECT_TYPE_VIBRATO_SOULSTUFF = 401,
	EFFECT_TYPE_VIBRATO_JITTER = 402,
    EFFECT_TYPE_VIBRATO_GRAFFITI = 403,
    
    EFFECT_TYPE_COLOR_MATRIX_BLACKNWHITE = 501,
    EFFECT_TYPE_COLOR_MATRIX_ESSENTIAL = 502,
    EFFECT_TYPE_COLOR_MATRIX_SUMMER = 503,
    
    EFFECT_TYPE_NONE = 5000,
} AliEffectType;




struct FloatFaceRect
{
    float x;
    float y;
    float width;
    float height;
};

struct FaceInputParam
{
    int srcWidth;
    int srcHeight;
    int srcRotate;
    string sFDModelPathName;
    string sLDModelNormalPathName;
    string sLDClassifierPathName;
};


struct EffectParams
{
    float fParams0; // [ for facebeauty, it's skin smooth param( default: 0.65f; range: 0 ~ 1 ) ]
    float fParams1; // [ for facebeauty, it's whtien param( default: 0.1f; range: 0 ~ 1 ) ]
    float fParams2; // [ for facebeauty, no use]
    float fParams3; // [ for facebeauty, it's whole face pink param( default: 0.15f; range: 0 ~ 1 ) ]
    float fParams4; // [ for facebeauty, it's shape hor param( default: 0.35f; range: 0 ~ 1 ) ]
    float fParams5; // [ for facebeauty, it's cheek pink param( default: 0.2f; range: 0 ~ 1 ) ]
    float fParams6; // [ for facebeauty, it's shape ver param( default: 0.0f; range: 0 ~ 1 ) ]
    float fParams7; // [ for facebeauty, it's shape eye param( default: 0.5f; range: 0 ~ 1 ) ]
    float fParams8; // [ for facebeauty, no use ]
    
    EffectParams()
    {
        fParams0 = 0.0f;
        fParams1 = 0.0f;
        fParams2 = 0.0f;
        fParams3 = 0.0f;
        fParams4 = 0.0f;
        fParams5 = 0.0f;
        fParams6 = 0.0f;
        fParams7 = 0.0f;
        fParams8 = 0.0f;
    }
};


#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */


#endif /* BaseStructDefine_h */
